@@ -48,6 +48,7 @@ local NOTIFY_LOBBY_ACCESS_DENIED = 'LOBBY-ACCESS-DENIED';
 local util = module:require "util";
 local ends_with = util.ends_with;
 local get_room_by_name_and_subdomain = util.get_room_by_name_and_subdomain;
+local internal_room_jid_match_rewrite = util.internal_room_jid_match_rewrite;
 local get_room_from_jid = util.get_room_from_jid;
 local is_healthcheck_room = util.is_healthcheck_room;
 local presence_check_status = util.presence_check_status;
@@ -254,6 +255,11 @@ function destroy_lobby_room(room, newjid, message)
             lobby_room_obj:destroy(newjid, message);
 
             module:log('info', 'Lobby room destroyed %s', lobby_room_obj.jid)
+
+            if room.jitsiMetadata then
+                room.jitsiMetadata.lobbyEnabled = false;
+                module:context(main_muc_component_config):fire_event('room-metadata-changed', { room = room; });
+            end
         end
         room._data.lobbyroom = nil;
         room._data.lobby_extra_reason = nil;
@@ -407,11 +413,16 @@ function process_lobby_muc_loaded(lobby_muc, host_module)
     host_module:hook('host-disco-info-node', function (event)
         local session, reply, node = event.origin, event.reply, event.node;
         if node == LOBBY_IDENTITY_TYPE
-            and session.jitsi_web_query_room
-            and check_display_name_required then
+            and session.jitsi_web_query_room then
             local room = get_room_by_name_and_subdomain(session.jitsi_web_query_room, session.jitsi_web_query_prefix);
 
-            if room and room._data.lobbyroom then
+            if room and room._data.lobby_disabled then
+                -- we cannot remove the child from the stanza so let's just change the type
+                local lobby_identity = reply:get_child_with_attr('identity', nil, 'type', LOBBY_IDENTITY_TYPE);
+                lobby_identity.attr.type = 'DISABLED_'..LOBBY_IDENTITY_TYPE;
+            end
+
+            if check_display_name_required and room and room._data.lobbyroom then
                 reply:tag('feature', { var = DISPLAY_NAME_REQUIRED_FEATURE }):up();
             end
         end
@@ -483,14 +494,26 @@ process_host_module(main_muc_component_config, function(host_module, host)
         end
         local members_only = event.fields['muc#roomconfig_membersonly'] and true or nil;
         if members_only then
+            -- if lobby disabled just ignore and return
+            if room._data.lobby_disabled then
+                module:log('warn', 'Lobby is disabled for room %s, cannot enable members only', room.jid);
+                return;
+            end
             local lobby_created = attach_lobby_room(room, actor);
             if lobby_created then
                 module:fire_event('jitsi-lobby-enabled', { room = room; });
                 event.status_codes['104'] = true;
                 notify_lobby_enabled(room, actor, true);
+
+                -- let's set it in the metadata and fire the event
+                if not room.jitsiMetadata then
+                    room.jitsiMetadata = {};
+                end
+                room.jitsiMetadata.lobbyEnabled = true;
+                host_module:fire_event('room-metadata-changed', { room = room; });
             end
         elseif room._data.lobbyroom then
-            destroy_lobby_room(room, room.jid);
+            destroy_lobby_room(room, internal_room_jid_match_rewrite(room.jid), nil); --
             module:fire_event('jitsi-lobby-disabled', { room = room; });
             notify_lobby_enabled(room, actor, false);
         end
@@ -500,7 +523,7 @@ process_host_module(main_muc_component_config, function(host_module, host)
         if room._data.lobbyroom then
             destroy_lobby_room(room, nil);
         end
-    end);
+    end, 1); -- prosody handles it at 0
     host_module:hook('muc-disco#info', function (event)
         local room = event.room;
         if (room._data.lobbyroom and room:get_members_only()) then
@@ -649,6 +672,13 @@ function handle_create_lobby(event)
     room._data.lobby_extra_reason = event.reason;
     room._data.lobby_skip_display_name_check = event.skip_display_name_check;
 
+    -- set in metadata without firing room-metadata-changed,
+    -- as this is a backend call and the caller will take care of that
+    if not room.jitsiMetadata then
+        room.jitsiMetadata = {};
+    end
+    room.jitsiMetadata.lobbyEnabled = true;
+
     -- Trigger a presence with 104 so existing participants retrieves new muc#roomconfig
     room:broadcast_message(
         st.message({ type='groupchat', from=room.jid })
